#include <FreeRTOS.h>
#include <aos/kernel.h>
#include <aos/yloop.h>
#include <bl_chip.h>
#include <bl_sec.h>
#include <bl_sys.h>
#include <bl_timer.h>
#include <bl_uart.h>
#include <cli.h>
#include <event_device.h>
#include <freertos_e907_helper_ps_with_irq.h>
#include <stdio.h>
#include <string.h>
#include <task.h>
#include <timers.h>
#include <vfs.h>
#ifdef FEATURE_ENABLE_EASYFLASH
#include <easyflash.h>
#endif
#include <bl60x_fw_api.h>
#include <bl808_glb.h>
#include <bl808_glb_gpio.h>
#include <bl808_pds.h>
#include <bl808_user_xram.h>
#include <bl_phy_api.h>
#include <boot_cpu0.h>
#include <core_rv32.h>
#include <fatfs.h>
#include <hal_sdh.h>
#include <hal_sys.h>
#include <hal_uart.h>
#include <hal_wifi.h>
#include <helper_audio.h>
#include <helper_sdh.h>
#include <helper_usb_cam.h>
#include <hosal_uart.h>
#include <lwip/err.h>
#include <lwip/netdb.h>
#include <lwip/sockets.h>
#include <lwip/tcp.h>
#include <lwip/tcpip.h>
#include <netutils/netutils.h>
#include <wifi_mgmr_ext.h>
#include <xram_peri_common.h>
#include <xram_peripheral.h>
#include <xram_platform.h>

#include "m1s_e907_xram.h"
#ifdef SYS_USER_VFS_ROMFS_ENABLE
#include <bl_romfs.h>
#include <fs/vfs_romfs.h>
#endif

#define TASK_AOS_TASKNAME ((char *)"aos_loop")
#define TASK_AOS_STACKSIZE (1024)
#define TASK_AOS_PRIORITY (15)
#define TASK_PHYRF_TASKNAME ((char *)"phyrf_loop")
#define TASK_PHYRF_STACKSIZE (1024)
#define TASK_PHYRF_PRIORITY (15)

#define WIFI_AP_PSM_INFO_SSID "conf_ap_ssid"
#define WIFI_AP_PSM_INFO_PASSWORD "conf_ap_psk"
#define WIFI_AP_PSM_INFO_PMK "conf_ap_pmk"
#define WIFI_AP_PSM_INFO_BSSID "conf_ap_bssid"
#define WIFI_AP_PSM_INFO_CHANNEL "conf_ap_channel"
#define WIFI_AP_PSM_INFO_IP "conf_ap_ip"
#define WIFI_AP_PSM_INFO_MASK "conf_ap_mask"
#define WIFI_AP_PSM_INFO_GW "conf_ap_gw"
#define WIFI_AP_PSM_INFO_DNS1 "conf_ap_dns1"
#define WIFI_AP_PSM_INFO_DNS2 "conf_ap_dns2"
#define WIFI_AP_PSM_INFO_IP_LEASE_TIME "conf_ap_ip_lease_time"
#define WIFI_AP_PSM_INFO_GW_MAC "conf_ap_gw_mac"
#define CLI_CMD_AUTOSTART1 "cmd_auto1"
#define CLI_CMD_AUTOSTART2 "cmd_auto2"
#define UART_PIN_TX_FOR_DEBUG (14)
#define UART_PIN_RX_FOR_DEBUG (15)

HOSAL_UART_DEV_DECL(uart_stdio, 0, UART_PIN_TX_FOR_DEBUG, UART_PIN_RX_FOR_DEBUG, 115200)

extern uint8_t _heap_start;
extern uint8_t _heap_size;  // @suppress("Type cannot be resolved")

static wifi_conf_t conf = {
    .country_code = "CN",
};
static wifi_interface_t wifi_interface;

HeapRegion_t xHeapRegions[] = {
    {&_heap_start, (uint32_t)&_heap_size},  // FIXME hardcode heap region set on runtime
    {NULL, 0}                               /* Terminates the array. */
};

void vApplicationStackOverflowHook(TaskHandle_t xTask, char *pcTaskName)
{
    puts("Stack Overflow checked\r\n");
    while (1) {
        /*empty here*/
    }
}

void vApplicationMallocFailedHook(void)
{
    // printf("Memory Allocate Failed. Current left size is %d bytes\r\n",
    //     xPortGetFreeHeapSize()
    // );
    while (1) {
        /*empty here*/
    }
}

void vApplicationIdleHook(void)
{
    __asm volatile("   wfi     ");
    /*empty*/
}

void vApplicationGetIdleTaskMemory(StaticTask_t **ppxIdleTaskTCBBuffer, StackType_t **ppxIdleTaskStackBuffer,
                                   uint32_t *pulIdleTaskStackSize)
{
    /* If the buffers to be provided to the Idle task are declared inside this
    function then they must be declared static - otherwise they will be allocated on
    the stack and so not exists after this function exits. */
    static StaticTask_t xIdleTaskTCB;
    static StackType_t uxIdleTaskStack[configMINIMAL_STACK_SIZE];

    /* Pass out a pointer to the StaticTask_t structure in which the Idle task's
    state will be stored. */
    *ppxIdleTaskTCBBuffer = &xIdleTaskTCB;

    /* Pass out the array that will be used as the Idle task's stack. */
    *ppxIdleTaskStackBuffer = uxIdleTaskStack;

    /* Pass out the size of the array pointed to by *ppxIdleTaskStackBuffer.
    Note that, as the array is necessarily of type StackType_t,
    configMINIMAL_STACK_SIZE is specified in words, not bytes. */
    *pulIdleTaskStackSize = configMINIMAL_STACK_SIZE;
}

/* configSUPPORT_STATIC_ALLOCATION and configUSE_TIMERS are both set to 1, so the
application must provide an implementation of vApplicationGetTimerTaskMemory()
to provide the memory that is used by the Timer service task. */
void vApplicationGetTimerTaskMemory(StaticTask_t **ppxTimerTaskTCBBuffer, StackType_t **ppxTimerTaskStackBuffer,
                                    uint32_t *pulTimerTaskStackSize)
{
    /* If the buffers to be provided to the Timer task are declared inside this
    function then they must be declared static - otherwise they will be allocated on
    the stack and so not exists after this function exits. */
    static StaticTask_t xTimerTaskTCB;
    static StackType_t uxTimerTaskStack[configTIMER_TASK_STACK_DEPTH];

    /* Pass out a pointer to the StaticTask_t structure in which the Timer
    task's state will be stored. */
    *ppxTimerTaskTCBBuffer = &xTimerTaskTCB;

    /* Pass out the array that will be used as the Timer task's stack. */
    *ppxTimerTaskStackBuffer = uxTimerTaskStack;

    /* Pass out the size of the array pointed to by *ppxTimerTaskStackBuffer.
    Note that, as the array is necessarily of type StackType_t,
    configTIMER_TASK_STACK_DEPTH is specified in words, not bytes. */
    *pulTimerTaskStackSize = configTIMER_TASK_STACK_DEPTH;
}

void user_vAssertCalled(void) __attribute__((weak, alias("vAssertCalled")));
void vAssertCalled(void)
{
    volatile uint32_t ulSetTo1ToExitFunction = 0;

    taskDISABLE_INTERRUPTS();
    while (ulSetTo1ToExitFunction != 1) {
        __asm volatile("NOP");
    }
}

static void _dump_boot_info(void)
{
    puts("------------------------------------------------------------\r\n");
    puts("Build Version: ");
    puts(BL_SDK_VER);  // @suppress("Symbol is not resolved")
    puts("\r\n");
    puts("Build Date: ");
    puts(__DATE__);
    puts("\r\n");
    puts("Build Time: ");
    puts(__TIME__);
    puts("\r\n");
    puts("------------------------------------------------------------\r\n");
}

static void system_thread_init()
{
    bl_sec_init();
    bl_sec_test();
    tcpip_init(NULL, NULL);
}

static void _cli_init()
{
    /*Put CLI which needs to be init here*/
    /* The following code is to make sure the cli command could be found by the linker */
#ifdef FEATURE_ENABLE_EASYFLASH
    easyflash_cli_init();
#endif
    network_netutils_iperf_cli_register();
    network_netutils_ping_cli_register();
    wifi_mgmr_cli_init();
    boot_cpu0_cli_init();
    helper_usb_cam_cli_init();
}

static void wifi_sta_connect(char *ssid, char *password)
{
    wifi_interface_t wifi_interface;

    wifi_interface = wifi_mgmr_sta_enable();
    wifi_mgmr_sta_connect(wifi_interface, ssid, password, NULL, NULL, 0, 0);
}

static unsigned char char_to_hex(char asccode)
{
    unsigned char ret;

    if ('0' <= asccode && asccode <= '9')
        ret = asccode - '0';
    else if ('a' <= asccode && asccode <= 'f')
        ret = asccode - 'a' + 10;
    else if ('A' <= asccode && asccode <= 'F')
        ret = asccode - 'A' + 10;
    else
        ret = 0;

    return ret;
}

static void _chan_str_to_hex(uint8_t *chan_band, uint16_t *chan_freq, char *chan)
{
    int i, freq_len, base = 1;
    uint8_t band;
    uint16_t freq = 0;
    char *p, *q;

    /*should have the following format
     * 2412|0
     * */
    p = strchr(chan, '|') + 1;
    if (NULL == p) {
        return;
    }
    band = char_to_hex(p[0]);
    (*chan_band) = band;

    freq_len = strlen(chan) - strlen(p) - 1;
    q = chan;
    q[freq_len] = '\0';
    for (i = 0; i < freq_len; i++) {
        freq = freq + char_to_hex(q[freq_len - 1 - i]) * base;
        base = base * 10;
    }
    (*chan_freq) = freq;
}

static void bssid_str_to_mac(uint8_t *hex, char *bssid, int len)
{
    unsigned char l4, h4;
    int i, lenstr;
    lenstr = len;

    if (lenstr % 2) {
        lenstr -= (lenstr % 2);
    }

    if (lenstr == 0) {
        return;
    }

    for (i = 0; i < lenstr; i += 2) {
        h4 = char_to_hex(bssid[i]);
        l4 = char_to_hex(bssid[i + 1]);
        hex[i / 2] = (h4 << 4) + l4;
    }
}
static void _connect_wifi()
{
    /*XXX caution for BIG STACK*/
    char pmk[66], bssid[32], chan[10];
    char ssid[33], password[66];
    char val_buf[66];
    // char val_len = sizeof(val_buf) - 1;
    uint8_t mac[6];
    uint8_t band = 0;
    uint16_t freq = 0;

    wifi_interface = wifi_mgmr_sta_enable();
    printf(
        "[APP] [WIFI] [T] %lld\r\n"
        "[APP]   Get STA %p from Wi-Fi Mgmr, pmk ptr %p, ssid ptr %p, password %p\r\n",
        aos_now_ms(), wifi_interface, pmk, ssid, password);
    memset(pmk, 0, sizeof(pmk));
    memset(ssid, 0, sizeof(ssid));
    memset(password, 0, sizeof(password));
    memset(bssid, 0, sizeof(bssid));
    memset(mac, 0, sizeof(mac));
    memset(chan, 0, sizeof(chan));

    memset(val_buf, 0, sizeof(val_buf));
    /*TODO no easy flash*/
    // ef_get_env_blob((const char *)WIFI_AP_PSM_INFO_SSID, val_buf, val_len, NULL);
    if (val_buf[0]) {
        /*We believe that when ssid is set, wifi_confi is OK*/
        strncpy(ssid, val_buf, sizeof(ssid) - 1);

        /*setup password ans PMK stuff from ENV*/
        memset(val_buf, 0, sizeof(val_buf));
        /*TODO no easy flash*/
        // ef_get_env_blob((const char *)WIFI_AP_PSM_INFO_PASSWORD, val_buf, val_len, NULL);
        if (val_buf[0]) {
            strncpy(password, val_buf, sizeof(password) - 1);
        }

        memset(val_buf, 0, sizeof(val_buf));
        /*TODO no easy flash*/
        // ef_get_env_blob((const char *)WIFI_AP_PSM_INFO_PMK, val_buf, val_len, NULL);
        if (val_buf[0]) {
            strncpy(pmk, val_buf, sizeof(pmk) - 1);
        }
        if (0 == pmk[0]) {
            printf("[APP] [WIFI] [T] %lld\r\n", aos_now_ms());
            puts("[APP]    Re-cal pmk\r\n");
            /*At lease pmk is not illegal, we re-cal now*/
            // XXX time consuming API, so consider lower-prirotiy for cal PSK to avoid sound glitch
            wifi_mgmr_psk_cal(password, ssid, strlen(ssid), pmk);
            /* TODO add easy flash*/
            // ef_set_env(WIFI_AP_PSM_INFO_PMK, pmk);
            // ef_save_env();
        }
        memset(val_buf, 0, sizeof(val_buf));
        /*TODO no easy flash*/
        // ef_get_env_blob((const char *)WIFI_AP_PSM_INFO_CHANNEL, val_buf, val_len, NULL);
        if (val_buf[0]) {
            strncpy(chan, val_buf, sizeof(chan) - 1);
            printf("connect wifi channel = %s\r\n", chan);
            _chan_str_to_hex(&band, &freq, chan);
        }
        memset(val_buf, 0, sizeof(val_buf));
        /*TODO no easy flash*/
        // ef_get_env_blob((const char *)WIFI_AP_PSM_INFO_BSSID, val_buf, val_len, NULL);
        if (val_buf[0]) {
            strncpy(bssid, val_buf, sizeof(bssid) - 1);
            printf("connect wifi bssid = %s\r\n", bssid);
            bssid_str_to_mac(mac, bssid, strlen(bssid));
            printf("mac = %02X:%02X:%02X:%02X:%02X:%02X\r\n", mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);
        }
        /* TODO no dts*/
        // } else if (0 == check_dts_config(ssid, password)) {
        /*nothing here*/
    } else {
        /*Won't connect, since ssid config is empty*/
        puts("[APP]    Empty Config\r\n");
        puts("[APP]    Try to set the following ENV with psm_set command, then reboot\r\n");
        puts("[APP]    NOTE: " WIFI_AP_PSM_INFO_PMK " MUST be psm_unset when conf is changed\r\n");
        puts("[APP]    env: " WIFI_AP_PSM_INFO_SSID "\r\n");
        puts("[APP]    env: " WIFI_AP_PSM_INFO_PASSWORD "\r\n");
        puts("[APP]    env(optinal): " WIFI_AP_PSM_INFO_PMK "\r\n");
        return;
    }

    printf(
        "[APP] [WIFI] [T] %lld\r\n"
        "[APP]    SSID %s\r\n"
        "[APP]    SSID len %d\r\n"
        "[APP]    password %s\r\n"
        "[APP]    password len %d\r\n"
        "[APP]    pmk %s\r\n"
        "[APP]    bssid %s\r\n"
        "[APP]    channel band %d\r\n"
        "[APP]    channel freq %d\r\n",
        aos_now_ms(), ssid, strlen(ssid), password, strlen(password), pmk, bssid, band, freq);
    // wifi_mgmr_sta_connect(wifi_interface, ssid, pmk, NULL);
    wifi_mgmr_sta_connect(wifi_interface, ssid, password, pmk, mac, band, freq);
}

static void event_cb_wifi_event(input_event_t *event, void *private_data)
{
    static char *ssid;
    static char *password;

    switch (event->code) {
        case CODE_WIFI_ON_INIT_DONE: {
            printf("[APP] [EVT] INIT DONE %lld\r\n", aos_now_ms());
            wifi_mgmr_start_background(&conf);
        } break;
        case CODE_WIFI_ON_MGMR_DONE: {
            printf("[APP] [EVT] MGMR DONE %lld, now %lums\r\n", aos_now_ms(), bl_timer_now_us() / 1000);
            _connect_wifi();
        } break;
        case CODE_WIFI_ON_MGMR_DENOISE: {
            printf("[APP] [EVT] Microwave Denoise is ON %lld\r\n", aos_now_ms());
        } break;
        case CODE_WIFI_ON_SCAN_DONE: {
            printf("[APP] [EVT] SCAN Done %lld\r\n", aos_now_ms());
            wifi_mgmr_cli_scanlist();
        } break;
        case CODE_WIFI_ON_SCAN_DONE_ONJOIN: {
            printf("[APP] [EVT] SCAN On Join %lld\r\n", aos_now_ms());
        } break;
        case CODE_WIFI_ON_DISCONNECT: {
            printf("[APP] [EVT] disconnect %lld, Reason: %s\r\n", aos_now_ms(),
                   wifi_mgmr_status_code_str(event->value));
        } break;
        case CODE_WIFI_ON_CONNECTING: {
            printf("[APP] [EVT] Connecting %lld\r\n", aos_now_ms());
        } break;
        case CODE_WIFI_CMD_RECONNECT: {
            printf("[APP] [EVT] Reconnect %lld\r\n", aos_now_ms());
        } break;
        case CODE_WIFI_ON_CONNECTED: {
#if 0
            uint32_t val;

            val = *(volatile uint32_t*)0x24b00060;
            val |= (1 << 1);
            *(volatile uint32_t*)0x24b00060 = val;
#endif

            printf("[APP] [EVT] connected %lld\r\n", aos_now_ms());
        } break;
        case CODE_WIFI_ON_PRE_GOT_IP: {
#if 0
            uint32_t val;

            val = *(volatile uint32_t*)0x24b00060;
            val |= (1 << 1);
            *(volatile uint32_t*)0x24b00060 = val;
#endif
            printf("[APP] [EVT] connected %lld\r\n", aos_now_ms());
        } break;
        case CODE_WIFI_ON_GOT_IP: {
            printf("[APP] [EVT] GOT IP %lld\r\n", aos_now_ms());
            printf("[SYS] Memory left is %d Bytes\r\n", xPortGetFreeHeapSize());
        } break;
        case CODE_WIFI_ON_EMERGENCY_MAC: {
            printf("[APP] [EVT] EMERGENCY MAC %lld\r\n", aos_now_ms());
            hal_reboot();  // one way of handling emergency is reboot. Maybe we should also consider solutions
        } break;
        case CODE_WIFI_ON_PROV_SSID: {
            printf("[APP] [EVT] [PROV] [SSID] %lld: %s\r\n", aos_now_ms(),
                   event->value ? (const char *)event->value : "UNKNOWN");
            if (ssid) {
                vPortFree(ssid);
                ssid = NULL;
            }
            ssid = (char *)event->value;
        } break;
        case CODE_WIFI_ON_PROV_BSSID: {
            printf("[APP] [EVT] [PROV] [BSSID] %lld: %s\r\n", aos_now_ms(),
                   event->value ? (const char *)event->value : "UNKNOWN");
            if (event->value) {
                vPortFree((void *)event->value);
            }
        } break;
        case CODE_WIFI_ON_PROV_PASSWD: {
            printf("[APP] [EVT] [PROV] [PASSWD] %lld: %s\r\n", aos_now_ms(),
                   event->value ? (const char *)event->value : "UNKNOWN");
            if (password) {
                vPortFree(password);
                password = NULL;
            }
            password = (char *)event->value;
        } break;
        case CODE_WIFI_ON_PROV_CONNECT: {
            printf("[APP] [EVT] [PROV] [CONNECT] %lld\r\n", aos_now_ms());
            printf("connecting to %s:%s...\r\n", ssid, password);
            wifi_sta_connect(ssid, password);
        } break;
        case CODE_WIFI_ON_PROV_DISCONNECT: {
            printf("[APP] [EVT] [PROV] [DISCONNECT] %lld\r\n", aos_now_ms());
        } break;
        case CODE_WIFI_ON_AP_STA_ADD: {
            printf("[APP] [EVT] [AP] [ADD] %lld, sta idx is %lu\r\n", aos_now_ms(), (uint32_t)event->value);
        } break;
        case CODE_WIFI_ON_AP_STA_DEL: {
            printf("[APP] [EVT] [AP] [DEL] %lld, sta idx is %lu\r\n", aos_now_ms(), (uint32_t)event->value);
        } break;
        default: {
            printf("[APP] [EVT] Unknown code %u, %lld\r\n", event->code, aos_now_ms());
            /*nothing*/
        }
    }
}

static void event_cb_cli_raw_event(input_event_t *event, void *private_data)
{
    switch (event->code) {
        case CODE_CLI_DATA_RAW: {
            // printf("CLI RAW Data, %s\r\n", (char *)event->value);
            aos_cli_input_direct((char *)event->value, strlen((char *)event->value));
            vPortFree((char *)event->value);
        } break;
        default: {
            /*nothing*/
        }
    }
}

static void app_delayed_action(void *arg)
{
    char *cmd1;
    struct {
        const char *cmd;
        int delay;
    } cmd_list[] = {
        {"c906\r\n", 10},
    };
    static unsigned int counter = 0;

    if (counter < sizeof(cmd_list) / sizeof(cmd_list[0])) {
        cmd1 = pvPortMalloc(strlen(cmd_list[counter].cmd));
        memcpy(cmd1, cmd_list[counter].cmd, strlen(cmd_list[counter].cmd));
        aos_post_event(EV_CLI, CODE_CLI_DATA_RAW, (unsigned long)cmd1);
        aos_post_delayed_action(cmd_list[counter].delay, app_delayed_action, NULL);
        counter++;
    }
}

static void aos_loop_proc(void *pvParameters)
{
    int fd_console;

    vfs_init();
    vfs_device_init();

    /* uart */
    vfs_uart_init_simple_mode(0, UART_PIN_TX_FOR_DEBUG, UART_PIN_RX_FOR_DEBUG, 2000000, "/dev/ttyS0");

    aos_loop_init();

    // romfs_register();
    // lfs_register();
    hal_sd_bus_4bits_enable();
    fatfs_register();
    helper_sdh_cli_init();
#ifdef FEATURE_ENABLE_EASYFLASH
    easyflash_init();
#endif

    fd_console = aos_open("/dev/ttyS0", 0);
    if (fd_console >= 0) {
        puts("Init CLI with event Driven\r\n");
        aos_cli_init(0);
        aos_poll_read_fd(fd_console, aos_cli_event_cb_read_get(), (void *)0x12345678);
        _cli_init();
        aos_register_event_filter(EV_CLI, event_cb_cli_raw_event, NULL);
        aos_post_delayed_action(10, app_delayed_action, NULL);
    }

    aos_register_event_filter(EV_WIFI, event_cb_wifi_event, NULL);

    puts("start aos loop... \r\n");

    aos_loop_run();

    puts("------------------------------------------\r\n");
    puts("+++++++++Critical Exit From Loop++++++++++\r\n");
    puts("******************************************\r\n");
    vTaskDelete(NULL);
}

static void cmd_stack_wifi(char *buf, int len, int argc, char **argv)
{
    /*wifi fw stack and thread stuff*/
    static uint8_t stack_wifi_init = 0;

    if (1 == stack_wifi_init) {
        puts("Wi-Fi Stack Started already!!!\r\n");
        return;
    }
    stack_wifi_init = 1;

    /* remove this usage when c89 */
    phy_powroffset_set((int8_t[4]){0x0, 0x0, 0x0, 0x0});
    bl_tpc_update_power_rate_11b((int8_t[4]){0x14, 0x14, 0x14, 0x12});
    bl_tpc_update_power_rate_11g((int8_t[8]){0x12, 0x12, 0x12, 0x12, 0x12, 0x12, 0xe, 0xe});
    bl_tpc_update_power_rate_11n((int8_t[8]){0x12, 0x12, 0x12, 0x12, 0x12, 0x10, 0xe, 0xe});

    printf("Start Wi-Fi fw @%lums\r\n", bl_timer_now_us() / 1000);
    hal_wifi_start_firmware_task();
    /*Trigger to start Wi-Fi*/
    printf("Start Wi-Fi fw is Done @%lums\r\n", bl_timer_now_us() / 1000);
    vTaskDelay(1000);
    aos_post_event(EV_WIFI, CODE_WIFI_ON_INIT_DONE, 0);
}

static void cmd_stack_mgmr(char *buf, int len, int argc, char **argv)
{
    aos_post_event(EV_WIFI, CODE_WIFI_ON_INIT_DONE, 0);
}

static void cmd_wifi(char *buf, int len, int argc, char **argv)
{
    void mm_sec_keydump(void);
    mm_sec_keydump();
}

// FIXME: 注释send_data_per_second函数后，uvc传输过程中e907跑飞
void send_data_per_second(int sockfd, struct sockaddr_in *client_addr, socklen_t *client_addr_len)
{
    char ReadBuff[50];
    while (1) {
        sendto(sockfd, ReadBuff, sizeof(ReadBuff), 0, (struct sockaddr *)client_addr, *client_addr_len);
    }
}

const static struct cli_command cmds_user[] STATIC_CLI_CMD_ATTRIBUTE = {
    {"stack_wifi", "Wi-Fi Stack", cmd_stack_wifi},
    {"stack_mgmr", "Wi-Fi Stack", cmd_stack_mgmr},
    {"wifi", "wifi", cmd_wifi},
};

void bfl_main()
{
    static StackType_t aos_loop_proc_stack[1024];
    static StaticTask_t aos_loop_proc_task;

    bl_uart_init(0, UART_PIN_TX_FOR_DEBUG, UART_PIN_RX_FOR_DEBUG, 255, 255, 2000000);

    printf("mtimer clock freq:%ld\r\n", CPU_Get_MTimer_Clock());

    bl_sys_lowlevel_init();
    _dump_boot_info();

    vPortDefineHeapRegions(xHeapRegions);

    puts("[OS] Starting aos_loop_proc task...\r\n");
    system_thread_init();

    xTaskCreateStatic(aos_loop_proc, TASK_AOS_TASKNAME, TASK_AOS_STACKSIZE, NULL, TASK_AOS_PRIORITY,
                      aos_loop_proc_stack, &aos_loop_proc_task);

    puts("[OS] Start e907 xram handle...\r\n");
    m1s_e907_xram_init();

    puts("[OS] Starting OS Scheduler...\r\n");
    vTaskStartScheduler();
}
